/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.page;

import org.primefaces.model.chart.AxisType;
import org.primefaces.model.chart.DateAxis;
import org.primefaces.model.chart.LegendPlacement;
import org.primefaces.model.chart.LineChartModel;
import org.primefaces.model.chart.LineChartSeries;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;
import ru.ulstu.service.TimeSeriesService;
import ru.ulstu.service.UtilService;

import javax.annotation.PostConstruct;
import javax.faces.view.ViewScoped;
import javax.inject.Named;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.ExecutionException;

@Named
@ViewScoped
public class IndexView implements Serializable {
    private final static Logger LOG = LoggerFactory.getLogger(IndexView.class);

    @Autowired
    private transient TimeSeriesService timeSeriesService;

    @Autowired
    private transient UtilService utilService;

    private LineChartModel model;
    private String timeSeriesString;

    @PostConstruct
    public void init() throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, ModelingException {
        timeSeriesString = utilService.getTimeSeriesToString(utilService.getRandomTimeSeries(50));
        createChart();
    }

    private LineChartModel initLinearModel() throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, ModelingException {
        LineChartModel model = new LineChartModel();

        LineChartSeries series1 = new LineChartSeries();
        series1.setLabel("Временной ряд");

        TimeSeries timeSeries = utilService.getTimeSeriesFromString(timeSeriesString);
        for (TimeSeriesValue value : timeSeries.getValues()) {
            series1.set(DateTimeFormatter.ISO_LOCAL_DATE.format(value.getDate()), value.getValue());
        }

        LineChartSeries series2 = new LineChartSeries();
        series2.setLabel("Сглаженный ряд");
        try {
            TimeSeries smoothedTimeSeries = timeSeriesService.smoothTimeSeries(timeSeries);
            for (TimeSeriesValue value : smoothedTimeSeries.getValues()) {
                series2.set(DateTimeFormatter.ISO_LOCAL_DATE.format(value.getDate()), value.getValue());
            }
        } catch (Exception ex) {
            LOG.warn(ex.getMessage());
        }
        LineChartSeries series3 = new LineChartSeries();
        series3.setLabel("Прогноз");
        TimeSeries forecast = timeSeriesService.getForecast(timeSeries, 20).getTimeSeries();
        for (TimeSeriesValue value : forecast.getValues()) {
            series3.set(DateTimeFormatter.ISO_LOCAL_DATE.format(value.getDate()), value.getValue());
        }
        model.addSeries(series1);
        model.addSeries(series2);
        model.addSeries(series3);
        return model;
    }

    public void createChart() throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, ModelingException {
        model = initLinearModel();
        model.setTitle("Сглаживание временного ряда");
        model.setLegendPosition("d");
        model.setLegendPlacement(LegendPlacement.OUTSIDEGRID);
        DateAxis xAxis = new DateAxis("Time");
        xAxis.setTickFormat("%#d %b %Y");
        model.getAxes().put(AxisType.X, xAxis);
    }

    public LineChartModel getModel() {
        return model;
    }

    public String getTimeSeriesString() throws ExecutionException, InterruptedException, InvocationTargetException, NoSuchMethodException, InstantiationException, IllegalAccessException, ModelingException {
        createChart();
        return timeSeriesString;
    }

    public void setTimeSeriesString(String timeSeriesString) {
        this.timeSeriesString = timeSeriesString;
    }
}
