/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.score;

import ru.ulstu.datamodel.Score;
import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;

import java.time.LocalDateTime;
import java.util.Map;

public abstract class ScoreMethod {
    private final String name;

    public ScoreMethod(String name) {
        this.name = name;
    }

    public Score getScore(Map<LocalDateTime, Double> tsValues, TimeSeries model) throws ModelingException {
        return new Score(this, evaluate(tsValues, model));
    }

    public abstract Number evaluate(Map<LocalDateTime, Double> tsValues, TimeSeries model) throws ModelingException;

    public String getName() {
        return name;
    }

    protected TimeSeriesValue getValueOnSameDate(TimeSeriesValue timeSeriesValueToFind, TimeSeries timeSeries) throws ModelingException {
        return timeSeries.getValues()
                .stream()
                .filter(v -> v.getDate().equals(timeSeriesValueToFind.getDate()))
                .findAny()
                .orElseThrow(() -> new ModelingException("Значение модельного ряда не найдено в оригинальном ряде: "
                        + timeSeriesValueToFind.getDate()
                        + " " + timeSeries));
    }
}
