/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.score;

import ru.ulstu.datamodel.exception.ModelingException;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;

import java.time.LocalDateTime;
import java.util.Map;
import java.util.Optional;

import static java.lang.Math.abs;

public class Smape extends ScoreMethod {
    public Smape() {
        super("Smape");
    }

    @Override
    public Number evaluate(Map<LocalDateTime, Double> tsValues, TimeSeries model) throws ModelingException {
        double sum = 0;
        for (TimeSeriesValue modelValue : model.getValues()) {
            //double actualValue = getValueOnSameDate(modelValue, original).getValue();
            double actualValue = Optional.ofNullable(tsValues.get(modelValue.getDate()))
                    .orElseThrow(() -> new ModelingException("Значение модельного ряда не найдено в оригинальном ряде: "
                            + modelValue.getDate()));
            sum += abs(modelValue.getValue() - actualValue)
                    / ((abs(actualValue) + abs(modelValue.getValue())) / 2);
        }
        sum = (double) Math.round(sum * 100) / 100;
        return sum * 100 / model.getLength();
    }
}