/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 *
 */

package ru.ulstu.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.ulstu.TimeSeriesUtils;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class UtilService {
    private static final Logger LOGGER = LoggerFactory.getLogger(UtilService.class);

    public TimeSeries getRandomTimeSeries(int length) {
        TimeSeries ts = new TimeSeries("Random time series");
        LocalDateTime dateStart = LocalDateTime.now().minusDays(length);
        for (int i = 0; i < length; i++) {
            ts.getValues().add(new TimeSeriesValue(dateStart, Math.random()));
            dateStart = dateStart.plusDays(1);
        }
        return ts;
    }

    public TimeSeries getTimeSeriesFromString(String tsString) {
        List<TimeSeriesValue> tsValues = Arrays.stream(tsString.split("\n"))
                .flatMap(v -> Arrays.stream(v.split(";")))
                .flatMap(v -> Arrays.stream(v.split(",")))
                .flatMap(v -> Arrays.stream(v.split("<br>")))
                .filter(v -> {
                    try {
                        Double.parseDouble(v);
                        return true;
                    } catch (NumberFormatException e) {
                        return false;
                    }
                })
                .map(Double::parseDouble)
                .map(TimeSeriesValue::new)
                .collect(Collectors.toList());
        return TimeSeriesUtils.fillDates(new TimeSeries(tsValues));
    }

    public String getTimeSeriesToString(TimeSeries timeSeries) {
        return timeSeries
                .getValues()
                .stream()
                .map(v -> v.getValue().toString().replace("\\.", ","))
                .collect(Collectors.joining(";"));
    }
}
